package org.apache.commons.lang3.builder;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.apache.commons.lang3.Validate;
import org.apache.commons.lang3.builder.Builder;
import org.apache.commons.lang3.builder.Diff;
import org.apache.commons.lang3.builder.DiffResult;
import org.apache.commons.lang3.builder.ToStringStyle;

public class DiffBuilder<T> implements Builder<DiffResult<T>> {
  private final List<Diff<?>> diffs;
  
  private final boolean objectsTriviallyEqual;
  
  private final T left;
  
  private final T right;
  
  private final ToStringStyle style;
  
  public DiffBuilder(T lhs, T rhs, ToStringStyle style, boolean testTriviallyEqual) {
    Validate.notNull(lhs, "lhs", new Object[0]);
    Validate.notNull(rhs, "rhs", new Object[0]);
    this.diffs = new ArrayList<>();
    this.left = lhs;
    this.right = rhs;
    this.style = style;
    this.objectsTriviallyEqual = (testTriviallyEqual && (lhs == rhs || lhs.equals(rhs)));
  }
  
  public DiffBuilder(T lhs, T rhs, ToStringStyle style) {
    this(lhs, rhs, style, true);
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, boolean lhs, boolean rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (lhs != rhs)
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, boolean[] lhs, boolean[] rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (!Arrays.equals(lhs, rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, byte lhs, byte rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (lhs != rhs)
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, byte[] lhs, byte[] rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (!Arrays.equals(lhs, rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, char lhs, char rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (lhs != rhs)
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, char[] lhs, char[] rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (!Arrays.equals(lhs, rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, double lhs, double rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (Double.doubleToLongBits(lhs) != Double.doubleToLongBits(rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, double[] lhs, double[] rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (!Arrays.equals(lhs, rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, float lhs, float rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (Float.floatToIntBits(lhs) != Float.floatToIntBits(rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, float[] lhs, float[] rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (!Arrays.equals(lhs, rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, int lhs, int rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (lhs != rhs)
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, int[] lhs, int[] rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (!Arrays.equals(lhs, rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, long lhs, long rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (lhs != rhs)
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, long[] lhs, long[] rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (!Arrays.equals(lhs, rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, short lhs, short rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (lhs != rhs)
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, short[] lhs, short[] rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (!Arrays.equals(lhs, rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, Object lhs, Object rhs) {
    Object objectToTest;
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (lhs == rhs)
      return this; 
    if (lhs != null) {
      objectToTest = lhs;
    } else {
      objectToTest = rhs;
    } 
    if (objectToTest.getClass().isArray()) {
      if (objectToTest instanceof boolean[])
        return append(fieldName, (boolean[])lhs, (boolean[])rhs); 
      if (objectToTest instanceof byte[])
        return append(fieldName, (byte[])lhs, (byte[])rhs); 
      if (objectToTest instanceof char[])
        return append(fieldName, (char[])lhs, (char[])rhs); 
      if (objectToTest instanceof double[])
        return append(fieldName, (double[])lhs, (double[])rhs); 
      if (objectToTest instanceof float[])
        return append(fieldName, (float[])lhs, (float[])rhs); 
      if (objectToTest instanceof int[])
        return append(fieldName, (int[])lhs, (int[])rhs); 
      if (objectToTest instanceof long[])
        return append(fieldName, (long[])lhs, (long[])rhs); 
      if (objectToTest instanceof short[])
        return append(fieldName, (short[])lhs, (short[])rhs); 
      return append(fieldName, (Object[])lhs, (Object[])rhs);
    } 
    if (lhs != null && lhs.equals(rhs))
      return this; 
    this.diffs.add(new Object(this, fieldName, lhs, rhs));
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, Object[] lhs, Object[] rhs) {
    validateFieldNameNotNull(fieldName);
    if (this.objectsTriviallyEqual)
      return this; 
    if (!Arrays.equals(lhs, rhs))
      this.diffs.add(new Object(this, fieldName, lhs, rhs)); 
    return this;
  }
  
  public org.apache.commons.lang3.builder.DiffBuilder<T> append(String fieldName, DiffResult<T> diffResult) {
    validateFieldNameNotNull(fieldName);
    Validate.notNull(diffResult, "diffResult", new Object[0]);
    if (this.objectsTriviallyEqual)
      return this; 
    for (Diff<?> diff : diffResult.getDiffs())
      append(fieldName + "." + diff.getFieldName(), diff
          .getLeft(), diff.getRight()); 
    return this;
  }
  
  public DiffResult<T> build() {
    return new DiffResult<>(this.left, this.right, this.diffs, this.style);
  }
  
  private void validateFieldNameNotNull(String fieldName) {
    Validate.notNull(fieldName, "fieldName", new Object[0]);
  }
}
